<TeXmacs|1.0.7.13>

<style|<tuple|article|twelf|doc|header-generic>>

<\body>
  \;

  <doc-data|<doc-title|Tm2elf User Guide>|<doc-author-data|<author-name|Tristan
  Crolard>>||<doc-running-title|nono>|<doc-date|<date>>>

  <\abstract>
    The <samp|Tm2elf> is a translator which converts a TeXmacs source file
    into Twelf syntax. The translator takes advantage of TeXmacs powerful
    macro system and XML-like native format to provide a user-friendly
    interface to Twelf combined with a high-quality pretty-printer. The tool,
    which is developed in Standard ML, is distributed as a .NET console
    application (the source code is available on request).
  </abstract>

  <samp|Tm2elf> is a translator which converts a TeXmacs source file into
  Twelf syntax. The source file should obey some very specific rules: it
  should only contain macro invocations. These macros are either provided by
  the <verbatim|twelf.ts> style file (which are prefixed by <verbatim|meta->
  and reflect Twelf syntax) or by another style file written by the user. The
  style files are of course needed by TeXmacs to display the source file
  content but are ignored by the translator.

  In this short guide, we explain how to write a TeXmacs source file
  containing grammars definitions, rules definitions ans meta-theorems with
  their proofs.

  <subsection|Grammars>

  As a first example, let us consider the following grammar for the
  <math|\<lambda\>>-calculus (using Twelf higher-order syntax):

  <meta-begin>

  <meta-decl#0|<expr>|<meta-type#2>>

  <meta-name|<expr>|E>

  \;

  <meta-decl#1|<e_app|<expr#1>|<expr#2>>|<expr#>>

  <meta-decl#2|<e_lam|<expr#x>|<expr#>>|<expr>>

  \;

  <meta-bind|1|2|<e_lam|<meta-wildcard>|<meta-wildcard>>>

  <meta-end><next-line>The above code may be obtained as follows:

  <\padded>
    <\inactive*>
      <meta-decl#0|<expr>|<meta-type#2>>

      <meta-name|<expr>|E>

      \;

      <meta-decl#1|<e_app|<expr#1>|<expr#2>>|<expr#>>

      <meta-decl#2|<e_lam|<expr#x>|<expr#>>|<expr>>

      \;

      <meta-bind|1|2|<e_lam|<meta-wildcard>|<meta-wildcard>>>
    </inactive*>
  </padded>

  The style file should contain the following macro-definitions:

  <\padded>
    <inactive|<assign|expr|<inactive|<macro|<math|<math-it|exp>>>>>>

    <inactive|<assign|expr#|<inactive|<macro|<math|E>>>>>

    <inactive|<assign|expr#1|<inactive|<macro|<math|E<rsub|1>>>>>>

    <inactive|<assign|expr#2|<inactive|<macro|<math|E<rsub|2>>>>>>

    <inactive|<assign|expr#x|<inactive|<macro|<math|x>>>>>

    \;

    <inactive|<assign|e_app|<macro|t|u|<math|<arg|t> <arg|u>>>>>

    <inactive|<assign|e_lam|<inactive|<macro|x|u|<math|\<lambda\><arg|x>.<arg|u>>>>>>
  </padded>

  Let us comment this example:

  <\itemize>
    <item>A typing declaration may have several forms. For instance, let us
    consider the binary application <samp|e_app>. The standard Twelf form is
    obtained as follows (where the wildcard symbol is required for each
    argument):

    <\padded>
      <meta-decl|<e_app|<meta-wildcard>|<meta-wildcard>>|<meta-imp|<expr>|<meta-imp|<expr>|<expr>>>>
    </padded>

    <no-indent>It is however also possible to declare:

    <\padded>
      <meta-decl|<e_app|<expr>|<expr>>|<expr>>
    </padded>

    <no-indent>and the translator will generate the former definition.

    <item>The <meta-bind|1|2|<e_lam|<meta-wildcard>|<meta-wildcard>>>
    specification instructs the converter to translate occurences of the form
    <samp|(e_lam x t)> into <samp|(e_lam [x]t)>. Binding specifications are
    also taken into account when translating declarations and:

    <\padded>
      <meta-decl|<e_lam|<expr>|<expr>>|<expr>>
    </padded>

    <no-indent>will be converted, as expected, into the same Twelf code as
    this declaration:

    <\padded>
      <meta-decl|<e_lam|<meta-wildcard>|<meta-wildcard>>|<meta-imp|<meta-paren|<meta-imp|<expr>|<expr>>>|<expr>>>
    </padded>

    <no-indent>Note that, since we edit directly the abstract syntax in
    TeXmacs, parentheses are never required for the translator. However, it
    is possible to display parentheses (as above) using <samp|meta-paren>.

    <item>A macro may have several variants. A variant is obtained by adding
    <verbatim|#> and a suffix (a number for instance) to its name. Variants
    are ignored by the translator (the suffix is removed) but they are useful
    for instance to display typing declarations as a grammar (as above).

    <item>Variables are uppercased by the translator when needed (by putting
    an underscore <verbatim|_> in front) and thus rule names begining with
    <verbatim|_> are forbidden (<verbatim|_> is however allowed <em|inside>
    rule names). Note also that mathematical subscript are also translated
    using <verbatim|_> and they are thus subjected to the same rule.

    <item>The Twelf code fragments should be enclosed within
    <verbatim|meta-begin> and <verbatim|meta-end> which are displayed on
    screen only (as angles).

    <item>Sections are allowed into Twelf code fragments and are turned into
    comments by the translator. Remarks and padded paragraphs are also
    allowed and ignored. This feature encourages a kind of literate Twelf
    programming by freely mixing text and code.
  </itemize>

  <subsection|Rules>

  <subsubsection|Judgments>

  Judgments should be declared using the <meta-type#1> variant of
  <inactive|<hybrid|meta-type>>. Binding specifications are also allowed for
  judgments.

  <subsubsection|Defining rules>

  A rule definition should be enclosed inside (and are displayed centered):

  <inactive*|<\meta-defn>
    \;
  </meta-defn>>

  Multiple hypotheses are obtained using <inactive*|<meta-hyps||>> (possibly
  nested many times for more than two hypotheses). For instance, here are the
  rules for evaluating <math|\<lambda\>>-terms:

  <\padded>
    <meta-decl#0|<eeval|<expr#1>|<expr#2>>|<meta-type#1>>

    <\meta-defn>
      <meta-rule||<eeval|<e_lam|x|<meta-app#0|e|x>>|<e_lam|x|<meta-app#0|e|x>>>|ev_lam>
    </meta-defn>

    <\meta-defn>
      <meta-rule|<meta-hyps|<eeval|e<rsub|1>|<e_lam|x|<meta-app#0|e<rsub|1><rprime|'>|x>>>|<meta-hyps|<eeval|e<rsub|2>|v<rsub|2>>|<eeval|<meta-app#0|e<rsub|1><rprime|'>|v<rsub|2>>|v>>>|<eeval|<e_app|e<rsub|1>|e<rsub|2>>|v>|ev_app>
    </meta-defn>

    \;

    <meta-mode|<eeval|+E<rsub|1>|-E<rsub|2>>>

    <meta-worlds|<meta-paren|><space|1em><eeval|E<rsub|1>|E<rsub|2>>>

    <meta-unique|<eeval|+E<rsub|1>|-1E<rsub|2>>>
  </padded>

  <subsubsection|Using rules in derivations>

  The rules are used using the same syntax as for declarations, but without
  the enclosing meta-definition. It is thus possible to copy a rule from its
  definition \ and paste it inside a derivation tree (in a proof for
  instance).

  <subsection|Lemmas>

  Lemmas should be declared using the <meta-type#0> variant of
  <inactive|<hybrid|meta-type>>. Binding specifications are also allowed for
  lemmas. A lemma is conveniently displayed for instance as follows (where
  ``name'' is the name of the lemma and formulas are separated by spaces,
  commas or conjunctions):

  <\equation*>
    \<varphi\><rsub|1><space|1em>\<ldots\><space|1em>\<varphi\><rsub|n><space|1em>\<vdash\><rsub|name><space|1em>\<psi\><rsub|1><space|1em>\<ldots\><space|1em>\<psi\><rsub|n>
  </equation*>

  The consistant mode declaration for this lemma would then be:

  <\padded>
    <meta-mode|+\<varphi\><rsub|1><space|1em>\<ldots\><space|1em>+\<varphi\><rsub|n><space|1em>\<vdash\><rsub|name><space|1em>-\<psi\><rsub|1><space|1em>\<ldots\><space|1em>-\<psi\><rsub|n>>
  </padded>

  Typed variables are also allowed and the should be enclosed in braces. For
  instance:

  <\padded>
    <meta-decl#0|<meta-braces#forall|e :exp><space|1em>\<vdash\><rsub|refl><space|1em>e=e|<meta-type#0>>
  </padded>

  As in Twelf, braces correspond to universal quantification when the mode is
  <math|+> and existential quantification when the mode is <math|-> (variants
  of <inactive|<hybrid|meta-braces>> for displaying quantifiers are
  available). Note also that braces should not be inserted when applying the
  lemma (but they are sometimes required for the statement of the lemma).\ 

  As an alternative, we could use a non-dependent existential quantifier (and
  make the mode declaration easier to read):

  <\padded>
    <meta-decl#0|<meta-braces#forall|e :exp><space|1em>\<vdash\><rsub|refl><space|1em><meta-braces#exists|\<cal-D\>
    :e=e>|<meta-type#0>>

    <meta-mode|<equal_ref|+e|-\<cal-D\>>>
  </padded>

  <subsubsection|Proofs>

  Let us consider the following algorithmic definition of equality (from
  [?]).

  \;

  <meta-decl#0|<tm_equal|<tm#1>|<tm#2>>|<meta-type#1>>

  \;

  <meta-defn|<meta-rule||<tm_equal|<single>|<single>>|equ_u>>

  <meta-defn|<meta-rule|<meta-all|x|<meta-imp|<tm_equal|x|x>|<tm_equal|<meta-app#0|t|x>|<meta-app#0|t<rprime|'>|x>>>>|<tm_equal|<lam|x|\<tau\>|<meta-app#0|t|x>>|<lam|x|\<tau\>|<meta-app#0|t<rprime|'>|x>>>|equ_l>>

  <meta-defn|<meta-rule|<meta-hyps|<tm_equal|t<rsub|1>|e<rsub|1>>|<tm_equal|t<rsub|2>|e<rsub|2>>>|<tm_equal|<meta-paren|<app|t<rsub|1>|t<rsub|2>>>|<meta-paren|<app|e<rsub|1>|e<rsub|2>>>>|equ_a>>

  \;

  Here is a proof (by induction on <math|t>) that equality is reflexive:

  \;

  <meta-decl#0|<equal_ref|<meta-braces#forall|<meta-typed-var|t|<tm>>>|<tm_equal|t|t>>|<meta-type#0>>

  <meta-mode|<equal_ref|+t|-\<cal-D\>>>

  \;

  <meta-defn|<meta-rule||<equal_ref|<meta-term#0|<tm>|<single>>|<meta-rule||<tm_equal|<single>|<single>>|equ_u>>|r_a>>

  \;

  \;

  <meta-defn|<meta-rule|<meta-all|x|<meta-all|<meta-typed-var|\<cal-U\>|<tm_equal|x|x>>|<meta-imp|<meta-paren|<equal_ref|<meta-term#0|<tm>|x>|<meta-term|<tm_equal|x|x>|\<cal-U\>>>>|<meta-paren|<equal_ref|<meta-term#0|<tm>|<meta-app#0|t|x>>|<meta-term|<tm_equal|<meta-app#0|t|x>|<meta-app#0|t|x>>|<meta-app|<meta-app|\<cal-D\>|x>|\<cal-U\>>>>>>>>|<equal_ref|<meta-term#0|<tm>|<lam|x|\<tau\>|<meta-app#0|t|x>>>|<meta-rule|<meta-term|<meta-all|x|<meta-imp|<tm_equal|x|x>|<tm_equal|<meta-app#0|t|x>|<meta-app#0|t|x>>>>|\<cal-D\>>|<tm_equal|<lam|x|\<tau\>|<meta-app#0|t|x>>|<lam|x|\<tau\>|<meta-app#0|t|x>>>|equ_l>>|r_l>>

  \;

  \;

  <meta-defn|<meta-rule|<meta-hyps|<meta-paren|<equal_ref|<meta-term#0|<tm>|t<rsub|1>>|<meta-term|<tm_equal|t<rsub|1>|t<rsub|1>>|\<cal-D\><rsub|1>>>>|<meta-paren|<equal_ref|<meta-term#0|<tm>|t<rsub|2>>|<meta-term|<tm_equal|t<rsub|2>|t<rsub|2>>|\<cal-D\><rsub|2>>>>>|<equal_ref|<meta-term#0|<tm>|<meta-paren|<app|t<rsub|1>|t<rsub|2>>>>|<meta-rule|<meta-hyps|<meta-term|<tm_equal|t<rsub|1>|t<rsub|1>>|\<cal-D\><rsub|1>>|<meta-term|<tm_equal|t<rsub|2>|t<rsub|2>>|\<cal-D\><rsub|2>>>|<tm_equal|<meta-paren|<app|t<rsub|1>|t<rsub|2>>>|<meta-paren|<app|t<rsub|1>|t<rsub|2>>>>|equ_a>>|r_a>>

  \;

  \;

  <meta-block-decl|l|<meta-block|<meta-braces|<meta-typed-var|x|<tm>>><meta-braces|<meta-typed-var|\<cal-U\>|<tm_equal|x|x>>><meta-braces|<meta-typed-var|r<rsub|x>|<meta-paren|<equal_ref|<meta-term#0|<tm>|x>|<meta-term#0|<tm_equal|x|x>|\<cal-U\>>>>>>|>>

  <meta-worlds|<meta-paren|l><space|1em><equal_ref|t|\<cal-D\>>>

  <meta-terminates|<meta-paren|t><space|1em><equal_ref|t|\<cal-D\>>>

  <meta-total|<meta-paren|t><space|1em><equal_ref|t|\<cal-D\>>>

  \;

  <subsection|Technical remarks>

  <\itemize>
    <item>Variables are uppercased by the translator when needed (by putting
    an underscore <verbatim|_> in front) and thus rule names begining with
    <verbatim|_> are forbidden (<verbatim|_> is however allowed <em|inside>
    rule names). Note also that mathematical subscript are also translated
    using <verbatim|_> and they are thus subjected to the same rule.

    <item>Twelf code fragments should be enclosed within
    <verbatim|meta-begin> and <verbatim|meta-end>. These macros are displayed
    on screen only (as angles). Otherwise the whole document is assumed to be
    a single Twelf code fragment.

    <item>Sections are allowed into Twelf code fragments and they are turned
    into comments by the translator. Remarks and padded paragraphs are also
    allowed and they are ignored. This feature encourages a kind of literate
    Twelf programming by freely mixing text and code.
  </itemize>

  More detailed informations are available in the <verbatim|twelf.ts> style
  file (given in appendix) and the provided examples.

  <appendix|The default Twelf style file>

  <include|/Users/crolard/TeXmacs/Twelf/twelf.ts>
</body>

<\initial>
  <\collection>
    <associate|font-base-size|10>
    <associate|page-screen-margin|false>
    <associate|par-first|0cm>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|1>>
    <associate|auto-2|<tuple|2|2>>
    <associate|auto-3|<tuple|2.1|2>>
    <associate|auto-4|<tuple|2.2|2>>
    <associate|auto-5|<tuple|2.3|3>>
    <associate|auto-6|<tuple|3|3>>
    <associate|auto-7|<tuple|3.1|3>>
    <associate|auto-8|<tuple|4|4>>
    <associate|auto-9|<tuple|A|4>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <with|par-left|<quote|1.5fn>|1<space|2spc>Grammars
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1>>

      <with|par-left|<quote|1.5fn>|2<space|2spc>Rules
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|3fn>|2.1<space|2spc>Judgments
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|3fn>|2.2<space|2spc>Defining rules
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|3fn>|2.3<space|2spc>Using rules in derivations
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>

      <with|par-left|<quote|1.5fn>|3<space|2spc>Lemmas
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-6>>

      <with|par-left|<quote|3fn>|3.1<space|2spc>Proofs
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-7>>

      <with|par-left|<quote|1.5fn>|4<space|2spc>Technical remarks
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-8>>

      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|Appendix
      A<space|2spc>The default Twelf style file>
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-9><vspace|0.5fn>
    </associate>
  </collection>
</auxiliary>